/*
 * Sonar, entreprise quality control tool.
 * Copyright (C) 2007-2008 Hortis-GRC SA
 * mailto:be_agile HAT hortis DOT ch
 *
 * Sonar is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Lesser General Public
 * License as published by the Free Software Foundation; either
 * version 3 of the License, or (at your option) any later version.
 *
 * Sonar is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * Lesser General Public License for more details.
 *
 * You should have received a copy of the GNU Lesser General Public
 * License along with {library}; if not, write to the Free Software
 * Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA  02
 */
package ch.hortis.sonar.model;

import org.apache.commons.lang.builder.*;

import javax.persistence.*;
import java.io.Serializable;
import java.util.*;

@Entity
@Table(name = "rules_profiles")
public class RulesProfile implements Serializable {
  public static final String SONAR_WAY_NAME = "Sonar way";
  public static final String SUN_CONVENTIONS_NAME = "Sun checks";

  @Id
  @Column(name = "id")
  @SequenceGenerator(name = "RULES_PROFILES_SEQ", sequenceName = "RULES_PROFILES_SEQ")
  @GeneratedValue(strategy = GenerationType.AUTO, generator = "RULES_PROFILES_SEQ")
  private Integer id;

  @Column(name = "name", updatable = true, nullable = false)
  private String name;

  @Column(name = "active", updatable = true, nullable = false)
  private Boolean active = Boolean.FALSE;
  
  @Column(name = "provided", updatable = true, nullable = false)
  private Boolean provided = Boolean.FALSE;

  @Column(name = "language", updatable = true, nullable = false)
  private String language;

  @OneToMany(mappedBy = "rulesProfile", fetch = FetchType.LAZY, cascade = {CascadeType.MERGE, CascadeType.PERSIST, CascadeType.REMOVE})
  private List<ActiveRule> activeRules;

  public RulesProfile() {
  }

  public RulesProfile(String name) {
    this.name=name;
    this.activeRules = new ArrayList<ActiveRule>();
  }

  public RulesProfile(String name, String language, boolean active, boolean provided) {
    this.name=name;
    this.language=language;
    this.active=active;
    this.provided=provided;
    this.activeRules = new ArrayList<ActiveRule>();
  }

  public Integer getId() {
    return id;
  }

  public void setId(Integer id) {
    this.id = id;
  }

  public String getName() {
    return name;
  }

  public void setName(String name) {
    this.name = name;
  }

  public List<ActiveRule> getActiveRules() {
    return activeRules;
  }

  public void setActiveRules(List<ActiveRule> activeRules) {
    this.activeRules = activeRules;
  }

  public Boolean getActive() {
    return active;
  }

  public void setActive(Boolean active) {
    this.active = active;
  }

  public Boolean getProvided() {
    return provided;
  }

  public void setProvided(Boolean provided) {
    this.provided = provided;
  }

  public String getLanguage() {
    return language;
  }

  public void setLanguage(String language) {
    this.language = language;
  }

  public List<ActiveRule> getActiveRules(RuleFailureLevel level) {
    List<ActiveRule> result = new ArrayList<ActiveRule>();
    for(ActiveRule activeRule:getActiveRules()) {
      if(activeRule.getLevel().equals(level)) {
        result.add(activeRule);
      }
    }
    return result;
  }

  public List<ActiveRule> getActiveRulesByPlugin(String pluginKey) {
    List<ActiveRule> result = new ArrayList<ActiveRule>();
    for(ActiveRule activeRule:getActiveRules()) {
      if(pluginKey.equals(activeRule.getPluginName())) {
        result.add(activeRule);
      }
    }
    return result;
  }

  public ActiveRule getActiveRule(String pluginKey, String ruleKey) {
    for(ActiveRule activeRule:getActiveRules()) {
      if(activeRule.getRuleKey().equals(ruleKey) && activeRule.getPluginName().equals(pluginKey)) {
        return activeRule;
      }
    }
    return null;
  }

  @Override
  public boolean equals(Object obj) {
    if (!(obj instanceof RulesProfile)) {
      return false;
    }
    if (this == obj) {
      return true;
    }
    RulesProfile other = (RulesProfile) obj;
    return new EqualsBuilder()
      .append(language, other.getLanguage())
      .append(name, other.getName())
      .isEquals();
  }

  @Override
  public int hashCode() {
    return new HashCodeBuilder(17, 37)
      .append(language)
      .append(name)
      .toHashCode();
  }

}
