/*
 * Sonar, entreprise quality control tool.
 * Copyright (C) 2007-2008 Hortis-GRC SA
 * mailto:be_agile HAT hortis DOT ch
 *
 * Sonar is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Lesser General Public
 * License as published by the Free Software Foundation; either
 * version 3 of the License, or (at your option) any later version.
 *
 * Sonar is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * Lesser General Public License for more details.
 *
 * You should have received a copy of the GNU Lesser General Public
 * License along with Sonar; if not, write to the Free Software
 * Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA  02
 */
package org.sonar.commons.database;

import org.hibernate.ScrollableResults;
import org.hibernate.SessionFactory;
import org.hibernate.StatelessSession;
import org.hibernate.Transaction;

public class BatchLoader {
  private DatabaseConnector connector;
  private StatelessSession statelessSession = null;
  private Transaction transaction = null;
  private ScrollableResults scroll = null;

  public BatchLoader(DatabaseConnector dc) {
    this.connector = dc;
  }

  public void begin(String hibernateQuery) {
    SessionFactory emf = (SessionFactory) connector.getEntityManagerFactory();
    statelessSession = emf.openStatelessSession();
    transaction = statelessSession.beginTransaction();
    scroll = statelessSession.createQuery(hibernateQuery).scroll();
  }

  public Object next() {
    if (scroll.next()) {
      return scroll.get(0);
    }
    return null;
  }

  // obj is loaded from the method next()
  public void update(Object obj) {
    statelessSession.update(obj);
  }

  public void commit() {
    transaction.commit();
    statelessSession.close();
  }
}